@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2014   The R Core Team.
+ *  Copyright (C) 1998-2019   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -190,9 +190,6 @@ static void R_restore_globals(RCNTXT *cptr)
        handling a stack overflow. */
     R_Expressions = R_Expressions_keep;
     R_BCNodeStackTop = cptr->nodestack;
-#ifdef BC_INT_STACK
-    R_BCIntStackTop = cptr->intstack;
-#endif
     R_Srcref = cptr->srcref;
 }
 
@@ -270,9 +267,6 @@ void begincontext(RCNTXT * cptr, int flags,
     cptr->restartstack = R_RestartStack;
     cptr->prstack = R_PendingPromises;
     cptr->nodestack = R_BCNodeStackTop;
-#ifdef BC_INT_STACK
-    cptr->intstack = R_BCIntStackTop;
-#endif
     cptr->srcref = R_Srcref;
     cptr->browserfinish = R_GlobalContext->browserfinish;
     cptr->nextcontext = R_GlobalContext;
@@ -776,6 +770,10 @@ Rboolean R_ToplevelExec(void (*fun)(void *), void *data)
     return result;
 }
 
+/* Return the current environment. */
+SEXP R_GetCurrentEnv() {
+    return R_GlobalContext->sysparent;
+}
 
 
 /*
